using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace howto_demonstrate_finally
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Run normally without errors.
        // Produces:    Statement
        //              Finally
        private void btnNormal_Click(object sender, EventArgs e)
        {
            lstResults.Items.Clear();
            try
            {
                // No error here.
                lstResults.Items.Add("Statement");
            }
            catch (Exception ex)
            {
                lstResults.Items.Add("Error: " + ex.Message);
            }
            finally
            {
                lstResults.Items.Add("Finally");
            }
        }

        // Use a break statement.
        // Produces:    Statement
        //              About to break
        //              Finally
        private void btnBreak_Click(object sender, EventArgs e)
        {
            lstResults.Items.Clear();

            for (; ; )
            {
                try
                {
                    // No error here.
                    lstResults.Items.Add("Statement");

                    // Break.
                    lstResults.Items.Add("About to break");
                    break;

                    // The compiler knows that the following code is unreachable.
                    lstResults.Items.Add("Never gets here");
                }
                catch (Exception ex)
                {
                    lstResults.Items.Add("Error: " + ex.Message);
                }
                finally
                {
                    lstResults.Items.Add("Finally");
                }
            }
        }

        // Use a continue statement.
        // Produces:    Statement
        //              About to continue
        //              Finally
        private void btnContinue_Click(object sender, EventArgs e)
        {
            lstResults.Items.Clear();

            do
            {
                try
                {
                    // No error here.
                    lstResults.Items.Add("Statement");

                    // Continue.
                    lstResults.Items.Add("About to continue");
                    continue;

                    // The compiler knows that the following code is unreachable.
                    lstResults.Items.Add("Never gets here");
                }
                catch (Exception ex)
                {
                    lstResults.Items.Add("Error: " + ex.Message);
                }
                finally
                {
                    lstResults.Items.Add("Finally");
                }

                lstResults.Items.Add("Never gets here");
            } while (false);
        }

        // Throw an error.
        // Produces:    Statement
        //              Error: Specified argument was out of the range of valid values.
        //              Finally
        private void btnError_Click(object sender, EventArgs e)
        {
            lstResults.Items.Clear();

            try
            {
                lstResults.Items.Add("Statement");

                // Throw an error.
                throw new ArgumentOutOfRangeException();

                // The compiler knows that the following code is unreachable.
                lstResults.Items.Add("Never gets here");
            }
            catch (Exception ex)
            {
                lstResults.Items.Add("Error: " + ex.Message);
            }
            finally
            {
                lstResults.Items.Add("Finally");
            }
        }

        // Return.
        // Produces:    Statement
        //              Finally
        private void btnReturn_Click(object sender, EventArgs e)
        {
            lstResults.Items.Clear();

            try
            {
                lstResults.Items.Add("Statement");

                // Return.
                return;

                // The compiler knows that the following code is unreachable.
                lstResults.Items.Add("Never gets here");
            }
            catch (Exception ex)
            {
                lstResults.Items.Add("Error: " + ex.Message);
            }
            finally
            {
                lstResults.Items.Add("Finally");
            }
        }

        // Throw an error and then another in the catch block.
        // Produces:    Statement
        //              Error: Specified argument was out of the range of valid values.
        //              Nested Error: Specified argument was out of the range of valid values.
        //              Nested Finally
        //              Finally
        private void btn2Errors_Click(object sender, EventArgs e)
        {
            lstResults.Items.Clear();

            try
            {
                lstResults.Items.Add("Statement");

                // Throw an error.
                throw new ArgumentOutOfRangeException();

                // The compiler knows that the following code is unreachable.
                lstResults.Items.Add("Never gets here");
            }
            catch (Exception ex)
            {
                lstResults.Items.Add("Error: " + ex.Message);

                // Throw another error.
                try
                {
                    throw new ArgumentOutOfRangeException();

                    // The compiler knows that the following code is unreachable.
                    lstResults.Items.Add("Never gets here");
                }
                catch (Exception inner_ex)
                {
                    lstResults.Items.Add("Nested Error: " + inner_ex.Message);
                }
                finally
                {
                    lstResults.Items.Add("Nested Finally");
                }
            }
            finally
            {
                lstResults.Items.Add("Finally");
            }
        }

        // Throw an error and return from within the catch block.
        // Produces:    Statement
        //              Error: Specified argument was out of the range of valid values.
        //              Finally
        private void btnErrorReturn_Click(object sender, EventArgs e)
        {
            lstResults.Items.Clear();

            try
            {
                lstResults.Items.Add("Statement");

                // Throw an error.
                throw new ArgumentOutOfRangeException();

                // The compiler knows that the following code is unreachable.
                lstResults.Items.Add("Never gets here");
            }
            catch (Exception ex)
            {
                lstResults.Items.Add("Error: " + ex.Message);

                // Return.
                return;

                // The compiler knows that the following code is unreachable.
                lstResults.Items.Add("Never gets here");
            }
            finally
            {
                lstResults.Items.Add("Finally");
            }
        }

        // Use Application.Exit to end the application.
        // This closes the window so this method displays
        // output in the console window.
        // Produces:    Statement
        //              The code actually gets here!
        //              Finally
        private void btnExit_Click(object sender, EventArgs e)
        {
            try
            {
                Console.WriteLine("Statement");

                // Exit.
                Application.Exit();

                Console.WriteLine("The code actually gets here!");
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: " + ex.Message);
            }
            finally
            {
                // The code does get here but the window
                // goes away so you can't see messages there.
                Console.WriteLine("Finally");
            }
        }

        // Use Environment.Exit to end the application.
        // This closes the window so this method displays
        // output in the console window.
        // Produces:    Statement
        private void btnEnvExit_Click(object sender, EventArgs e)
        {
            try
            {
                Console.WriteLine("Statement");

                // Exit.
                Environment.Exit(0);

                Console.WriteLine("The code actually gets here!");
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: " + ex.Message);
            }
            finally
            {
                // The code does get here but the window
                // goes away so you can't see messages there.
                Console.WriteLine("Finally");
            }
        }
    }
}
